@@ -1110,7 +1110,7 @@ void svt_av1_highbd_warp_affine_sse4_1(const int32_t *mat, const uint8_t *ref8b,
     assert(IMPLIES(conv_params->do_average, conv_params->is_compound));
 
     const int     offset_bits_vert       = bd + 2 * FILTER_BITS - reduce_bits_horiz;
-    const __m128i clip_pixel             = _mm_set1_epi16(bd == 10 ? 1023 : (bd == 12 ? 4095 : 255));
+    const __m128i clp_pxl                = _mm_set1_epi16(bd == 10 ? 1023 : (bd == 12 ? 4095 : 255));
     const __m128i reduce_bits_vert_shift = _mm_cvtsi32_si128(reduce_bits_vert);
     const __m128i reduce_bits_vert_const = _mm_set1_epi32(((1 << reduce_bits_vert) >> 1));
     const __m128i res_add_const          = _mm_set1_epi32(1 << offset_bits_vert);
@@ -1348,7 +1348,7 @@ void svt_av1_highbd_warp_affine_sse4_1(const int32_t *mat, const uint8_t *ref8b,
                         res32_lo         = _mm_sra_epi32(_mm_add_epi32(res32_lo, round_bits_const), round_bits_shift);
 
                         __m128i res16_lo = _mm_packus_epi32(res32_lo, res32_lo);
-                        res16_lo         = _mm_min_epi16(res16_lo, clip_pixel);
+                        res16_lo         = _mm_min_epi16(res16_lo, clp_pxl);
                         _mm_storel_epi64(dst16, res16_lo);
                     } else {
                         res_lo = _mm_packus_epi32(res_lo, res_lo);
@@ -1373,7 +1373,7 @@ void svt_av1_highbd_warp_affine_sse4_1(const int32_t *mat, const uint8_t *ref8b,
                             __m128i res32_hi = _mm_add_epi32(res_hi, res_sub_const);
                             res32_hi = _mm_sra_epi32(_mm_add_epi32(res32_hi, round_bits_const), round_bits_shift);
                             __m128i res16_hi = _mm_packus_epi32(res32_hi, res32_hi);
-                            res16_hi         = _mm_min_epi16(res16_hi, clip_pixel);
+                            res16_hi         = _mm_min_epi16(res16_hi, clp_pxl);
                             _mm_storel_epi64(dst16_4, res16_hi);
                         } else {
                             res_hi = _mm_packus_epi32(res_hi, res_hi);
