@@ -55,6 +55,70 @@ static EbErrorType rtime_alloc_palette_tokens(SequenceControlSet *scs, PictureCo
     }
     return EB_ErrorNone;
 }
+// Min-Heap Utilities
+
+// Swap two elements in the heap
+static inline void heap_swap(uint64_t *heap, int i, int j) {
+    uint64_t t = heap[i];
+    heap[i]    = heap[j];
+    heap[j]    = t;
+}
+
+// Restore heap property upward (after insertion)
+static void heap_sift_up(uint64_t *heap, int index) {
+    while (index > 0) {
+        int parent = (index - 1) >> 1;
+        if (heap[parent] <= heap[index])
+            break;
+        heap_swap(heap, parent, index);
+        index = parent;
+    }
+}
+
+// Restore heap property downward (after removal)
+static void heap_sift_down(uint64_t *heap, int count, int index) {
+    for (;;) {
+        int left  = (index << 1) + 1;
+        int right = left + 1;
+        int min   = index;
+
+        if (left < count && heap[left] < heap[min])
+            min = left;
+        if (right < count && heap[right] < heap[min])
+            min = right;
+
+        if (min == index)
+            break;
+
+        heap_swap(heap, index, min);
+        index = min;
+    }
+}
+
+// Get smallest element (top of heap) without removing it
+static inline uint64_t heap_min(const uint64_t *heap, int count) { return (count > 0) ? heap[0] : UINT64_MAX; }
+// Remove and return smallest element (top of heap)
+static inline uint64_t heap_pop_min(uint64_t *heap, int *count_io) {
+    int      count = *count_io;
+    uint64_t val   = heap[0];
+    heap[0]        = heap[--count];
+    heap_sift_down(heap, count, 0);
+    *count_io = count;
+    return val;
+}
+
+// Insert new value into heap
+// Returns 1 on success, 0 if heap is full (no growth)
+static inline bool heap_push(uint64_t *heap, int *count_io, int capacity, uint64_t value) {
+    int count = *count_io;
+    if (count == capacity)
+        return false; // heap full, drop or handle overflow
+    heap[count] = value;
+    heap_sift_up(heap, count);
+    *count_io = count + 1;
+    return true;
+}
+
 extern MvReferenceFrame svt_get_ref_frame_type(uint8_t list, uint8_t ref_idx);
 
 void svt_aom_largest_coding_unit_dctor(EbPtr p);
@@ -82,16 +146,14 @@ EbErrorType svt_aom_picture_manager_context_ctor(EbThreadContext *thread_ctx, co
     context_ptr->picture_manager_output_fifo_ptr = svt_system_resource_get_producer_fifo(
         enc_handle_ptr->rate_control_tasks_resource_ptr, rate_control_index);
     context_ptr->picture_control_set_fifo_ptr = svt_system_resource_get_producer_fifo(
-        enc_handle_ptr->picture_control_set_pool_ptr_array[0], 0); //The Child PCS Pool here
-    context_ptr->recon_coef_fifo_ptr = svt_system_resource_get_producer_fifo(enc_handle_ptr->enc_dec_pool_ptr_array[0],
+        enc_handle_ptr->picture_control_set_pool_ptr, 0); //The Child PCS Pool here
+    context_ptr->recon_coef_fifo_ptr = svt_system_resource_get_producer_fifo(enc_handle_ptr->enc_dec_pool_ptr,
                                                                              0); //The Child PCS Pool here
 
     context_ptr->consecutive_dec_order = 0;
     EB_MALLOC_ARRAY(context_ptr->started_pics_dec_order, ppcs_count);
-    context_ptr->started_pics_dec_order_size     = ppcs_count;
-    context_ptr->started_pics_dec_order_head_idx = 0;
-    context_ptr->started_pics_dec_order_tail_idx = 0;
-
+    context_ptr->started_pics_dec_order_size  = ppcs_count;
+    context_ptr->started_pics_dec_order_count = 0;
     return EB_ErrorNone;
 }
 
@@ -222,8 +284,9 @@ void superres_setup_child_pcs(SequenceControlSet *entry_scs_ptr, PictureParentCo
                                              (uint16_t)sb_index,
                                              child_pcs->enc_mode,
                                              entry_scs_ptr->static_config.rtc,
-                                             entry_scs_ptr->static_config.screen_content_mode,
                                              entry_scs_ptr->max_block_cnt,
+                                             entry_scs_ptr->allintra,
+                                             entry_scs_ptr->input_resolution,
                                              child_pcs);
             // Increment the Order in coding order (Raster Scan Order)
             sb_origin_y = (sb_origin_x == pic_width_in_sb - 1) ? sb_origin_y + 1 : sb_origin_y;
@@ -337,8 +400,8 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             // Post the Full Results Object
             svt_post_full_object(out_results_wrapper);
 
-            pcs     = (PictureParentControlSet *)NULL;
-            enc_ctx = (EncodeContext *)NULL;
+            pcs     = NULL;
+            enc_ctx = NULL;
             break;
         }
         case EB_PIC_INPUT:
@@ -359,9 +422,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
 
             // Overlay pics should be NREF
             if (pcs->is_ref) {
-#if OPT_LD_LATENCY2
                 svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
                 for (uint32_t i = 0; i < enc_ctx->ref_pic_list_length; i++) {
                     ref_entry = enc_ctx->ref_pic_list[i];
                     if (!ref_entry->is_valid) {
@@ -373,7 +434,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                         i != enc_ctx->ref_pic_list_length - 1, enc_ctx->app_callback_ptr, EB_ENC_PM_ERROR5);
                 }
                 ref_entry->picture_number                = pcs->picture_number;
-                ref_entry->reference_object_ptr          = (EbObjectWrapper *)NULL;
+                ref_entry->reference_object_ptr          = NULL;
                 ref_entry->release_enable                = true;
                 ref_entry->reference_available           = false;
                 ref_entry->slice_type                    = pcs->slice_type;
@@ -384,16 +445,14 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                 ref_entry->is_ref                        = pcs->is_ref;
                 ref_entry->decode_order                  = pcs->decode_order;
                 ref_entry->refresh_frame_mask            = pcs->av1_ref_signal.refresh_frame_mask;
-                ref_entry->dec_order_of_last_ref         = pcs->is_ref ? UINT64_MAX : 0;
+                ref_entry->dec_order_of_last_ref         = UINT64_MAX;
                 ref_entry->frame_end_cdf_update_required = pcs->frame_end_cdf_update_mode;
 
                 CHECK_REPORT_ERROR(
-                    (pcs->pred_struct_ptr->pred_struct_period * REF_LIST_MAX_DEPTH < MAX_ELAPSED_IDR_COUNT),
+                    (pcs->pred_struct_ptr->pred_struct_entry_count * REF_LIST_MAX_DEPTH < MAX_ELAPSED_IDR_COUNT),
                     enc_ctx->app_callback_ptr,
                     EB_ENC_PM_ERROR6);
-#if OPT_LD_LATENCY2
                 svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             }
             break;
 
@@ -401,9 +460,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             scs     = input_pic_demux->scs;
             enc_ctx = scs->enc_ctx;
             // Find the Reference in the Reference List
-#if OPT_LD_LATENCY2
             svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             for (uint32_t i = 0; i < enc_ctx->ref_pic_list_length; i++) {
                 ref_entry = enc_ctx->ref_pic_list[i];
                 if (ref_entry->is_valid && ref_entry->picture_number == input_pic_demux->picture_number) {
@@ -420,18 +477,14 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             CHECK_REPORT_ERROR((ref_entry->picture_number == input_pic_demux->picture_number),
                                enc_ctx->app_callback_ptr,
                                EB_ENC_PM_ERROR8);
-#if OPT_LD_LATENCY2
             svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             break;
         case EB_PIC_FEEDBACK:
             scs     = input_pic_demux->scs;
             enc_ctx = scs->enc_ctx;
 
             // Find the Reference in the Reference Queue
-#if OPT_LD_LATENCY2
             svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             for (uint32_t i = 0; i < enc_ctx->ref_pic_list_length; i++) {
                 ref_entry = enc_ctx->ref_pic_list[i];
                 if (ref_entry->is_valid && ref_entry->picture_number == input_pic_demux->picture_number) {
@@ -443,9 +496,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                 // Sometimes the reference may not be in the queue (e.g. if a delayed-I causes a ref
                 // pic to not be needed, it may be dropped from the queue before feedback arrives).
             }
-#if OPT_LD_LATENCY2
             svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             // Update the last decode order
             if (input_pic_demux->decode_order == decode_order)
                 decode_order++;
@@ -456,8 +507,8 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
 
             CHECK_REPORT_ERROR_NC(enc_ctx->app_callback_ptr, EB_ENC_PM_ERROR9);
 
-            pcs     = (PictureParentControlSet *)NULL;
-            enc_ctx = (EncodeContext *)NULL;
+            pcs     = NULL;
+            enc_ctx = NULL;
 
             break;
         }
@@ -465,7 +516,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
         // ***********************************
         //  Common Code
         // *************************************
-        if (enc_ctx == (EncodeContext *)NULL) {
+        if (enc_ctx == NULL) {
             // If no enc_ctx, release the Input Picture Demux Results and exit
             svt_release_object(input_pic_demux_wrapper);
             continue;
@@ -509,22 +560,17 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                     // hardcode the reference for the overlay frame
                     uint64_t ref_poc = entry_ppcs->is_overlay ? entry_ppcs->picture_number
                                                               : entry_ppcs->av1_ref_signal.ref_poc_array[ref];
-#if OPT_LD_LATENCY2
                     svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
                     ref_entry = search_ref_in_ref_queue(enc_ctx, ref_poc);
 
                     refs_available = (ref_entry == NULL) ? false
-                        : (scs->static_config.rate_control_mode && entry_ppcs->slice_type != I_SLICE &&
-                           entry_ppcs->temporal_layer_index == 0 && !ref_entry->feedback_arrived &&
-                           !enc_ctx->terminating_sequence_flag_received)
+                        : (scs->static_config.rate_control_mode && entry_ppcs->temporal_layer_index == 0 &&
+                           !ref_entry->feedback_arrived && !enc_ctx->terminating_sequence_flag_received)
                         ? false
-                        : (entry_ppcs->frame_end_cdf_update_mode && !ref_entry->frame_context_updated) ? false
-                        : (ref_entry->reference_available) ? true // The Reference has been completed
-                                                           : false; // The Reference has not been completed
-#if OPT_LD_LATENCY2
+                        : (entry_ppcs->frame_end_cdf_update_mode && !ref_entry->frame_context_updated)
+                        ? false
+                        : ref_entry->reference_available;
                     svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
                 }
             }
 
@@ -609,42 +655,35 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             child_pcs->enc_dec_coded_sb_count = 0;
             child_pcs->hbd_md                 = entry_ppcs->hbd_md;
             context_ptr->pmgr_dec_order       = child_pcs->ppcs->decode_order;
+            // Update consecutive_dec_order using the min-heap of pending decode orders.
+            // When the next expected picture arrives, consecutive_dec_order is advanced
+            // and any immediately following orders are popped from the heap.
+            // Otherwise, the current decode_order is stored in the heap until the gap is filled.
+            uint64_t *decode_order_heap = context_ptr->started_pics_dec_order;
+            int      *heap_n            = &context_ptr->started_pics_dec_order_count;
 
-            // Update the consecutive decode order count, if this picture is the next
-            // picture in decode order. Otherwise, add the picture to the
-            // started_pics_dec_order list so the consecutive decode order count can be
-            // properly updated later.
             if (entry_ppcs->decode_order == context_ptr->consecutive_dec_order + 1) {
+                // Next expected picture arrived
                 context_ptr->consecutive_dec_order++;
-
-                if (context_ptr->started_pics_dec_order_head_idx != context_ptr->started_pics_dec_order_tail_idx) {
-                    for (int idx = context_ptr->started_pics_dec_order_head_idx;
-                         idx != context_ptr->started_pics_dec_order_tail_idx;) {
-                        if (context_ptr->started_pics_dec_order[idx] == context_ptr->consecutive_dec_order + 1) {
-                            context_ptr->consecutive_dec_order++;
-                            idx = context_ptr->started_pics_dec_order_head_idx;
-                        } else {
-                            idx = (idx == context_ptr->started_pics_dec_order_size - 1) ? 0 : idx + 1;
-                        }
-                    }
-
-                    /* clang-format off */
-                    while (context_ptr->started_pics_dec_order_head_idx != context_ptr->started_pics_dec_order_tail_idx &&
-                        context_ptr->started_pics_dec_order[context_ptr->started_pics_dec_order_head_idx] <= context_ptr->consecutive_dec_order) {
-                        context_ptr->started_pics_dec_order_head_idx =
-                            (context_ptr->started_pics_dec_order_head_idx == context_ptr->started_pics_dec_order_size - 1)
-                            ? 0
-                            : context_ptr->started_pics_dec_order_head_idx + 1;
-                    }
-                    /* clang-format on */
+                // If we update the consecutive_dec_order, then we should check all pictures in the pic_mgr_input_pic_list
+                // to see if they can be started. This is necessary to avoid a hang. Consider the case where picture B is
+                // stored in pic_mgr_input_pic_list in a spot after picture A. It is possible that picture B could be started
+                // before A and update the consecutive_dec_order. If there are no reference or input frames that occur
+                // after starting picture B (hence pic manager receives no inputs to re-check the loop) then picture A would
+                // never be started. This of course assumes picture B updates the consecutive_dec_order to some value that is
+                // required by picture A to start. An alternative solution to prevent the hang would be to order pic_mgr_input_pic_list
+                // based on decode order, in which case re-iterating through the list would not be necessary, as the previously
+                // described scenario could not happen.
+                input_list_idx = 0;
+                // Consume consecutive values already waiting in the heap
+                while (*heap_n > 0 && heap_min(decode_order_heap, *heap_n) == context_ptr->consecutive_dec_order + 1) {
+                    heap_pop_min(decode_order_heap, heap_n);
+                    context_ptr->consecutive_dec_order++;
                 }
-            } else if (entry_ppcs->decode_order > 0) {
-                context_ptr->started_pics_dec_order[context_ptr->started_pics_dec_order_tail_idx] =
-                    entry_ppcs->decode_order;
-                context_ptr->started_pics_dec_order_tail_idx = (context_ptr->started_pics_dec_order_tail_idx ==
-                                                                context_ptr->started_pics_dec_order_size - 1)
-                    ? 0
-                    : context_ptr->started_pics_dec_order_tail_idx + 1;
+            } else if (entry_ppcs->decode_order > context_ptr->consecutive_dec_order) {
+                // Out-of-order arrival, push directly
+                heap_push(
+                    decode_order_heap, heap_n, context_ptr->started_pics_dec_order_size, entry_ppcs->decode_order);
             }
             // 3.make all  init for ChildPCS
             uint16_t pic_width_in_sb = (entry_ppcs->aligned_width + entry_scs_ptr->sb_size - 1) /
@@ -681,8 +720,9 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                                                      (uint16_t)sb_index,
                                                      child_pcs->enc_mode,
                                                      scs->static_config.rtc,
-                                                     scs->static_config.screen_content_mode,
                                                      scs->max_block_cnt,
+                                                     scs->allintra,
+                                                     scs->input_resolution,
                                                      child_pcs);
                     // Increment the Order in coding order (Raster Scan Order)
                     sb_origin_y = (sb_origin_x == pic_width_in_sb - 1) ? sb_origin_y + 1 : sb_origin_y;
@@ -740,14 +780,10 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             }
             cm->mi_stride = child_pcs->mi_stride;
             // Reset the Reference Lists
-            EB_MEMSET(child_pcs->ref_pic_ptr_array[REF_LIST_0], 0, REF_LIST_MAX_DEPTH * sizeof(EbObjectWrapper *));
-            EB_MEMSET(child_pcs->ref_pic_ptr_array[REF_LIST_1], 0, REF_LIST_MAX_DEPTH * sizeof(EbObjectWrapper *));
-            EB_MEMSET(child_pcs->ref_pic_qp_array[REF_LIST_0], 0, REF_LIST_MAX_DEPTH * sizeof(uint8_t));
-            EB_MEMSET(child_pcs->ref_pic_qp_array[REF_LIST_1], 0, REF_LIST_MAX_DEPTH * sizeof(uint8_t));
-            EB_MEMSET(child_pcs->ref_slice_type_array[REF_LIST_0], 0, REF_LIST_MAX_DEPTH * sizeof(SliceType));
-            EB_MEMSET(child_pcs->ref_slice_type_array[REF_LIST_1], 0, REF_LIST_MAX_DEPTH * sizeof(SliceType));
-            EB_MEMSET(child_pcs->ref_pic_r0[REF_LIST_0], 0, REF_LIST_MAX_DEPTH * sizeof(double));
-            EB_MEMSET(child_pcs->ref_pic_r0[REF_LIST_1], 0, REF_LIST_MAX_DEPTH * sizeof(double));
+            svt_memset(child_pcs->ref_pic_ptr_array, 0, sizeof(child_pcs->ref_pic_ptr_array));
+            svt_memset(child_pcs->ref_pic_qp_array, 0, sizeof(child_pcs->ref_pic_qp_array));
+            svt_memset(child_pcs->ref_slice_type_array, 0, sizeof(child_pcs->ref_slice_type_array));
+            svt_memset(child_pcs->ref_pic_r0, 0, sizeof(child_pcs->ref_pic_r0));
             int8_t ref_index = 0;
             if (entry_ppcs->slice_type == B_SLICE) {
                 int8_t max_temporal_index = -1;
@@ -838,10 +874,8 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
             // Post the Full Results Object
             svt_post_full_object(out_results_wrapper);
             // Remove the Input Entry from the Input Queue
-            input_entry->input_object_ptr = (EbObjectWrapper *)NULL;
-#if OPT_LD_LATENCY2
+            input_entry->input_object_ptr = NULL;
             svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
             for (uint32_t i = 0; i < enc_ctx->ref_pic_list_length; i++) {
                 ref_entry = enc_ctx->ref_pic_list[i];
                 if (ref_entry->is_valid) {
@@ -852,14 +886,10 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                     }
                 }
             }
-#if OPT_LD_LATENCY2
             svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
         }
 
-#if OPT_LD_LATENCY2
         svt_block_on_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
         for (uint32_t i = 0; i < enc_ctx->ref_pic_list_length; i++) {
             ref_entry = enc_ctx->ref_pic_list[i];
             if (ref_entry->is_valid) {
@@ -869,7 +899,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                     (!ref_entry->frame_end_cdf_update_required || ref_entry->frame_context_updated)) {
                     // Release the nominal live_count value
                     svt_release_object(ref_entry->reference_object_ptr);
-                    ref_entry->reference_object_ptr  = (EbObjectWrapper *)NULL;
+                    ref_entry->reference_object_ptr  = NULL;
                     ref_entry->reference_available   = false;
                     ref_entry->is_ref                = false;
                     ref_entry->is_valid              = false;
@@ -879,9 +909,7 @@ void *svt_aom_picture_manager_kernel(void *input_ptr) {
                 }
             }
         }
-#if OPT_LD_LATENCY2
         svt_release_mutex(enc_ctx->ref_pic_list_mutex);
-#endif
 
         // Release the Input Picture Demux Results
         svt_release_object(input_pic_demux_wrapper);
